import { __decorate, __param } from 'tslib';
import { InjectionToken, Input, HostBinding, Component, ViewChild, ViewContainerRef, ElementRef, Renderer2, Optional, Inject, ChangeDetectorRef, ComponentFactoryResolver, ContentChild, TemplateRef, Directive, ContentChildren, Injectable, Pipe, NgModule } from '@angular/core';
import { FormControlName, FormArrayName, FormGroup, ControlContainer, FormGroupDirective, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { Subject, merge } from 'rxjs';
import { debounceTime } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

const ValidationErrorsConfigObject = new InjectionToken('ValidationErrorsConfigObject');
const FOR_ROOT_OPTIONS_TOKEN = new InjectionToken('forRootOptionToken');
const MESSAGES_PROVIDER = new InjectionToken('MessagesProvider');
const MESSAGES_PIPE_FACTORY_TOKEN = new InjectionToken('MessagePipeFactoryToken');

let InputErrorsComponent = class InputErrorsComponent {
    constructor() {
        this.messages = [];
        this.params = {};
    }
};
__decorate([
    Input()
], InputErrorsComponent.prototype, "messages", void 0);
__decorate([
    Input()
], InputErrorsComponent.prototype, "params", void 0);
__decorate([
    Input(), HostBinding('class.is-inner')
], InputErrorsComponent.prototype, "innerValidationError", void 0);
__decorate([
    Input()
], InputErrorsComponent.prototype, "control", void 0);
InputErrorsComponent = __decorate([
    Component({
        // tslint:disable-next-line:component-selector
        selector: 'input-errors',
        template: `
      <div class="error-wrapper">
          <span
              class="help-block"
              *ngFor="let message of messages;"
          >{{ message | mapToMessage: params }}</span>
      </div>
  `,
        styles: [":host{position:absolute}:host .error-wrapper{position:relative;left:41px;bottom:2px}:host .error-wrapper span{font-size:.8em;color:red;font-weight:500}:host(.is-inner){position:absolute}:host(.is-inner) .error-wrapper{position:relative;left:37px;bottom:20px}:host(.is-inner) .error-wrapper span{font-size:.8em;color:red;font-weight:500}"]
    })
], InputErrorsComponent);

function toScreamingSnakeCase(input) {
    return input.replace(/([a-z])([A-Z])/g, '$1_$2').toUpperCase();
}

function wrapMethod(subject$, name, control) {
    const prevMethod = control[name];
    function wrappedMethod(...args) {
        prevMethod.bind(control)(...args);
        subject$.next();
    }
    control[name] = wrappedMethod;
}
function toChangeObservable(control) {
    const touchedChanges$ = new Subject();
    wrapMethod(touchedChanges$, 'markAsTouched', control);
    wrapMethod(touchedChanges$, 'markAsUntouched', control);
    wrapMethod(touchedChanges$, 'markAsDirty', control);
    wrapMethod(touchedChanges$, 'markAsPristine', control);
    const obs = merge(touchedChanges$, control.statusChanges);
    return obs.pipe(debounceTime(100));
}

class FormValidationContainer {
    constructor(elRef, renderer, messageProvider, cdRef, componentFactoryResolver, validationErrorsConfig) {
        this.elRef = elRef;
        this.renderer = renderer;
        this.messageProvider = messageProvider;
        this.cdRef = cdRef;
        this.componentFactoryResolver = componentFactoryResolver;
        this.validationErrorsConfig = validationErrorsConfig;
        this.customErrorMessages = {};
        this.messageParams = {};
        this.validationDisabled = false;
        this.validationContext = validationErrorsConfig.defaultContext;
    }
    ngAfterContentInit() {
        this.addErrorComponent();
        this.subscription = toChangeObservable(this.formControl).subscribe(value => {
            this.checkErrors();
            this.checkSuccess();
            this.updateErrorComponent();
        });
    }
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
    addErrorComponent() {
        if (this.errorsContainer && !this.componentRef) {
            this.errorsContainer.clear();
            const componentFactory = this.componentFactoryResolver.resolveComponentFactory(this.validationErrorsConfig.errorComponent);
            this.componentRef = this.errorsContainer.createComponent(componentFactory);
        }
    }
    updateErrorComponent() {
        if (this.componentRef) {
            this.componentRef.instance.innerValidationError = this.innerValidationError;
            this.componentRef.instance.messages = this.messages;
            this.componentRef.instance.params = this.messageParams;
        }
    }
    checkErrors() {
        const hasError = (!this.formControl.valid && this.formControl.dirty && this.formControl.touched) && !this.validationDisabled;
        if (hasError && this.el && this.el.nativeElement) {
            this.messages = Object.keys(this.formControl.errors).map(error => {
                const fieldName = this.formControlName;
                const errorKey = `${toScreamingSnakeCase(fieldName + '')}.ERRORS.${toScreamingSnakeCase(error)}`;
                if (this.messageProvider &&
                    this.messageProvider.instant(`${this.validationContext}.${errorKey}`) === `${this.validationContext}.${errorKey}`) {
                    return `${this.validationErrorsConfig.defaultContext}.ERRORS.${toScreamingSnakeCase(error)}`;
                }
                else {
                    return `${this.validationContext}.${errorKey}`;
                }
            });
            const params = Object.values(this.formControl.errors).reduce((a, b) => {
                a = Object.assign(Object.assign({}, a), b);
                return a;
            }, {});
            this.messageParams = this.messageParams ? Object.assign(Object.assign({}, this.messageParams), params) : params;
            if (this.messages && this.messages.length > 0) {
                this.messages = [this.messages[0]];
            }
            try {
                this.renderer.removeClass(this.el.nativeElement, 'is-valid');
            }
            catch (e) {
            }
            this.renderer.addClass(this.el.nativeElement, 'is-invalid');
        }
        this.hasErrors = hasError;
    }
    checkSuccess() {
        const hasSuccess = (this.formControl.valid &&
            this.formControl.dirty && this.formControl.touched) &&
            !this.validationDisabled;
        if (hasSuccess && this.el && this.el.nativeElement) {
            this.messages = [];
            try {
                this.renderer.removeClass(this.el.nativeElement, 'is-invalid');
            }
            catch (e) {
            }
        }
        this.hasSuccess = hasSuccess;
    }
    setValidationContext(context) {
        this.validationContext = context;
    }
    setInnerValidation(innerValidation) {
        this.innerValidationError = innerValidation;
    }
    clear() {
        this.formControl.reset();
        this.formControl.setErrors(null);
        this.renderer.removeClass(this.el.nativeElement, 'is-valid');
        this.renderer.removeClass(this.el.nativeElement, 'is-invalid');
        this.messages = [];
    }
}
__decorate([
    Input()
], FormValidationContainer.prototype, "customErrorMessages", void 0);
__decorate([
    Input()
], FormValidationContainer.prototype, "messageParams", void 0);
__decorate([
    Input()
], FormValidationContainer.prototype, "validationDisabled", void 0);
__decorate([
    Input()
], FormValidationContainer.prototype, "innerValidationError", void 0);
__decorate([
    ViewChild('errorsContainer', { read: ViewContainerRef, static: true })
], FormValidationContainer.prototype, "errorsContainer", void 0);
__decorate([
    HostBinding('class.has-error')
], FormValidationContainer.prototype, "hasErrors", void 0);
__decorate([
    HostBinding('class.has-success')
], FormValidationContainer.prototype, "hasSuccess", void 0);

const VALIDATION_ERROR_CONFIG = new InjectionToken('validationError.config');

let FormFieldContainerComponent = class FormFieldContainerComponent extends FormValidationContainer {
    constructor(
    // tslint:disable-next-line:variable-name
    _elRef, 
    // tslint:disable-next-line:variable-name
    _renderer, 
    // tslint:disable-next-line:variable-name
    _messageProvider, 
    // tslint:disable-next-line:variable-name
    _cdRef, 
    // tslint:disable-next-line:variable-name
    _componentFactoryResolver, 
    // tslint:disable-next-line:variable-name
    _validationErrorsConfig) {
        super(_elRef, _renderer, _messageProvider, _cdRef, _componentFactoryResolver, _validationErrorsConfig);
        this._elRef = _elRef;
        this._renderer = _renderer;
        this._messageProvider = _messageProvider;
        this._cdRef = _cdRef;
        this._componentFactoryResolver = _componentFactoryResolver;
        this._validationErrorsConfig = _validationErrorsConfig;
    }
    get formControl() {
        return this._formControl.control;
    }
    get formControlName() {
        return this._formControl.name;
    }
    get statusChanges() {
        return this._formControl.control.statusChanges;
    }
    get el() {
        return this._input;
    }
};
FormFieldContainerComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [MESSAGES_PROVIDER,] }] },
    { type: ChangeDetectorRef },
    { type: ComponentFactoryResolver },
    { type: undefined, decorators: [{ type: Inject, args: [VALIDATION_ERROR_CONFIG,] }] }
];
__decorate([
    ContentChild(FormControlName, { static: true })
], FormFieldContainerComponent.prototype, "_formControl", void 0);
__decorate([
    ContentChild(FormControlName, { read: ElementRef, static: true })
], FormFieldContainerComponent.prototype, "_input", void 0);
FormFieldContainerComponent = __decorate([
    Component({
        selector: '[ngxValidationErrorsField], ngx-validation-errors-field, [formFieldContainer], form-field-container',
        template: `
      <ng-content></ng-content>
      <ng-template #errorsContainer></ng-template>
  `
    }),
    __param(2, Optional()), __param(2, Inject(MESSAGES_PROVIDER)),
    __param(5, Inject(VALIDATION_ERROR_CONFIG))
], FormFieldContainerComponent);

let FormArrayContainerComponent = class FormArrayContainerComponent extends FormValidationContainer {
    constructor(
    // tslint:disable-next-line:variable-name
    _elRef, 
    // tslint:disable-next-line:variable-name
    _renderer, 
    // tslint:disable-next-line:variable-name
    _messageProvider, 
    // tslint:disable-next-line:variable-name
    _cdRef, 
    // tslint:disable-next-line:variable-name
    _componentFactoryResolver, 
    // tslint:disable-next-line:variable-name
    _validationErrorsConfig) {
        super(_elRef, _renderer, _messageProvider, _cdRef, _componentFactoryResolver, _validationErrorsConfig);
        this._elRef = _elRef;
        this._renderer = _renderer;
        this._messageProvider = _messageProvider;
        this._cdRef = _cdRef;
        this._componentFactoryResolver = _componentFactoryResolver;
        this._validationErrorsConfig = _validationErrorsConfig;
    }
    get formControl() {
        return this._formControl.control;
    }
    get formControlName() {
        return this._formControl.name;
    }
    get statusChanges() {
        return this._formControl.control.statusChanges;
    }
    get el() {
        return this._el;
    }
};
FormArrayContainerComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [MESSAGES_PROVIDER,] }] },
    { type: ChangeDetectorRef },
    { type: ComponentFactoryResolver },
    { type: undefined, decorators: [{ type: Inject, args: [VALIDATION_ERROR_CONFIG,] }] }
];
__decorate([
    ContentChild(FormArrayName, { static: true })
], FormArrayContainerComponent.prototype, "_formControl", void 0);
__decorate([
    ContentChild(FormArrayName, { read: ElementRef, static: true })
], FormArrayContainerComponent.prototype, "_el", void 0);
FormArrayContainerComponent = __decorate([
    Component({
        selector: '[ngxValidationErrorsArray], ngx-validation-errors-array, [formArrayContainer], form-array-container',
        template: `
      <ng-content></ng-content>
      <ng-container #errorsContainer></ng-container>
  `
    }),
    __param(2, Optional()), __param(2, Inject(MESSAGES_PROVIDER)),
    __param(5, Inject(VALIDATION_ERROR_CONFIG))
], FormArrayContainerComponent);

class ForFieldErrorsContext {
    constructor(errors) {
        this.errors = errors;
    }
}
let FormFieldEmptyContainerDirective = class FormFieldEmptyContainerDirective {
    constructor(renderer, viewContainer, template, 
    // @Optional() form: FormGroupDirective,
    messageProvider, validationErrorsConfig) {
        this.renderer = renderer;
        this.viewContainer = viewContainer;
        this.template = template;
        this.messageProvider = messageProvider;
        this.validationErrorsConfig = validationErrorsConfig;
        this.customErrorMessages = {};
        this.messageParams = {};
        this.validationDisabled = false;
        this.context = { errors: [] };
        this.validationContext = validationErrorsConfig.defaultContext;
        const nodes = this.viewContainer.createEmbeddedView(this.template, this.context);
        this.rootEl = nodes.rootNodes[0];
    }
    ngDoCheck() {
        const hasError = (!this.formControl.valid && this.formControl.touched) && !this.validationDisabled;
        let messages;
        if (hasError) {
            messages = Object.keys(this.formControl.errors).map(error => {
                const fieldName = this.formControlName;
                const errorKey = `${toScreamingSnakeCase(fieldName)}.ERRORS.${toScreamingSnakeCase(error)}`;
                if (this.messageProvider &&
                    this.messageProvider.instant(`${this.validationContext}.${errorKey}`) === `${this.validationContext}.${errorKey}`) {
                    return `${this.validationErrorsConfig.defaultContext}.ERRORS.${toScreamingSnakeCase(error)}`;
                }
                else {
                    return `${this.validationContext}.${errorKey}`;
                }
            });
            const params = Object.values(this.formControl.errors).reduce((a, b) => {
                a = Object.assign(Object.assign({}, a), b);
                return a;
            }, {});
            this.messageParams = this.messageParams ? Object.assign(Object.assign({}, this.messageParams), params) : params;
            if (messages && messages.length > 0) {
                messages = [messages[0]];
            }
        }
        if ((messages && !this.messages) || (!messages && this.messages) || (messages && messages[0] !== this.messages[0])) {
            this.messages = messages;
            this.context.errors = messages;
            if (this.rootEl) {
                if (messages) {
                    this.renderer.addClass(this.rootEl, 'has-error');
                }
                else {
                    this.renderer.removeClass(this.rootEl, 'has-error');
                }
            }
        }
    }
    setValidationContext(context) {
        this.validationContext = context;
    }
    setInnerValidation(innerValidation) {
    }
    clear() {
        this.formControl.reset();
        this.formControl.setErrors(null);
        this.messages = [];
        this.context.errors = undefined;
    }
    get formControl() {
        return this.formControlRef;
    }
    get formControlName() {
        if (this.formControlRef['_parent'] instanceof FormGroup) {
            const form = this.formControlRef['_parent'];
            const name = Object.keys(form.controls).find(k => form.controls[k] === this.formControlRef);
            return name;
        }
        return 'field';
    }
};
FormFieldEmptyContainerDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ViewContainerRef },
    { type: TemplateRef },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [MESSAGES_PROVIDER,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [VALIDATION_ERROR_CONFIG,] }] }
];
__decorate([
    Input('ngxValidationErrors')
], FormFieldEmptyContainerDirective.prototype, "formControlRef", void 0);
__decorate([
    Input()
], FormFieldEmptyContainerDirective.prototype, "customErrorMessages", void 0);
__decorate([
    Input()
], FormFieldEmptyContainerDirective.prototype, "messageParams", void 0);
__decorate([
    Input()
], FormFieldEmptyContainerDirective.prototype, "validationDisabled", void 0);
FormFieldEmptyContainerDirective = __decorate([
    Directive({
        selector: '[ngxValidationErrors]',
        providers: [
            {
                provide: ControlContainer,
                useExisting: FormGroupDirective
            }
        ]
    }),
    __param(3, Optional()), __param(3, Inject(MESSAGES_PROVIDER)),
    __param(4, Inject(VALIDATION_ERROR_CONFIG))
], FormFieldEmptyContainerDirective);

let ValidationContextComponent = class ValidationContextComponent {
    constructor(cdRef) {
        this.cdRef = cdRef;
    }
    ngAfterContentInit() {
        if (this.fieldValidators) {
            this.fieldValidators.forEach(i => {
                i.setValidationContext(this.validationContext);
                i.setInnerValidation(this.innerValidationError);
            });
        }
        if (this.arrayValidators) {
            this.arrayValidators.forEach(i => {
                i.setValidationContext(this.validationContext);
                i.setInnerValidation(this.innerValidationError);
            });
        }
        if (this.directives) {
            this.directives.forEach(i => {
                i.setValidationContext(this.validationContext);
                i.setInnerValidation(this.innerValidationError);
            });
        }
    }
    clear() {
        if (this.fieldValidators) {
            this.fieldValidators.forEach(v => {
                v.clear();
            });
        }
        if (this.arrayValidators) {
            this.arrayValidators.forEach(v => {
                v.clear();
            });
        }
        if (this.directives) {
            this.directives.forEach(v => {
                v.clear();
            });
        }
        this.cdRef.markForCheck();
    }
};
ValidationContextComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
__decorate([
    ContentChildren(FormFieldContainerComponent, { descendants: true })
], ValidationContextComponent.prototype, "fieldValidators", void 0);
__decorate([
    ContentChildren(FormArrayContainerComponent, { descendants: true })
], ValidationContextComponent.prototype, "arrayValidators", void 0);
__decorate([
    ContentChildren(FormFieldEmptyContainerDirective, { descendants: true })
], ValidationContextComponent.prototype, "directives", void 0);
__decorate([
    Input()
], ValidationContextComponent.prototype, "validationContext", void 0);
__decorate([
    Input()
], ValidationContextComponent.prototype, "innerValidationError", void 0);
ValidationContextComponent = __decorate([
    Component({
        // tslint:disable:component-selector
        selector: '[validationContext]',
        template: '<ng-content></ng-content>'
    })
], ValidationContextComponent);

let InnerMapToMessagePipe = class InnerMapToMessagePipe {
    transform(value, args) {
        return value;
    }
};
InnerMapToMessagePipe = __decorate([
    Injectable(),
    Pipe({
        name: 'innerMapToMessage',
        pure: false
    })
], InnerMapToMessagePipe);

let MapToMessagePipe = class MapToMessagePipe {
    constructor(cdRef, pipeFactory) {
        this.pipeFactory = pipeFactory;
        if (pipeFactory) {
            try {
                this.pipe = pipeFactory(cdRef);
            }
            catch (e) {
                console.error(e);
                this.pipe = new InnerMapToMessagePipe();
            }
        }
        else {
            this.pipe = new InnerMapToMessagePipe();
        }
    }
    transform(value, ...args) {
        return this.pipe.transform(value, ...args);
    }
};
MapToMessagePipe.ctorParameters = () => [
    { type: ChangeDetectorRef, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [MESSAGES_PIPE_FACTORY_TOKEN,] }] }
];
MapToMessagePipe = __decorate([
    Pipe({
        name: 'mapToMessage',
        pure: false
    }),
    __param(0, Optional()),
    __param(1, Optional()), __param(1, Inject(MESSAGES_PIPE_FACTORY_TOKEN))
], MapToMessagePipe);

var NgxValidationErrorsModule_1;
const defaultConfig = {
    defaultContext: 'GENERAL',
    errorComponent: InputErrorsComponent
};
function configFactory(customConfig, currentConfig) {
    const actualConfig = Object.assign({}, currentConfig);
    if (customConfig) {
        if (customConfig.defaultContext) {
            actualConfig.defaultContext = customConfig.defaultContext;
        }
        if (customConfig.errorComponent) {
            actualConfig.errorComponent = customConfig.errorComponent;
        }
    }
    return actualConfig;
}
let NgxValidationErrorsModule = NgxValidationErrorsModule_1 = class NgxValidationErrorsModule {
    static forRoot(config) {
        return {
            ngModule: NgxValidationErrorsModule_1,
            providers: [
                {
                    provide: ValidationErrorsConfigObject, useValue: defaultConfig
                },
                {
                    provide: FOR_ROOT_OPTIONS_TOKEN,
                    useValue: config
                },
                MapToMessagePipe,
                {
                    provide: VALIDATION_ERROR_CONFIG,
                    useFactory: configFactory,
                    deps: [FOR_ROOT_OPTIONS_TOKEN, ValidationErrorsConfigObject]
                },
                MapToMessagePipe,
                InnerMapToMessagePipe
            ]
        };
    }
};
NgxValidationErrorsModule = NgxValidationErrorsModule_1 = __decorate([
    NgModule({
        declarations: [
            InputErrorsComponent,
            FormFieldContainerComponent,
            FormFieldEmptyContainerDirective,
            FormArrayContainerComponent,
            ValidationContextComponent,
            MapToMessagePipe,
            InnerMapToMessagePipe
        ],
        imports: [
            CommonModule,
            FormsModule,
            ReactiveFormsModule
        ],
        exports: [
            InputErrorsComponent,
            FormFieldContainerComponent,
            FormArrayContainerComponent,
            ValidationContextComponent,
            FormFieldEmptyContainerDirective
        ],
        entryComponents: [
            InputErrorsComponent
        ]
    })
], NgxValidationErrorsModule);

/**
 * Generated bundle index. Do not edit.
 */

export { ForFieldErrorsContext, FormArrayContainerComponent, FormFieldContainerComponent, FormFieldEmptyContainerDirective, InputErrorsComponent, MESSAGES_PIPE_FACTORY_TOKEN, MESSAGES_PROVIDER, NgxValidationErrorsModule, ValidationContextComponent, defaultConfig as ɵa, configFactory as ɵb, ValidationErrorsConfigObject as ɵc, FOR_ROOT_OPTIONS_TOKEN as ɵd, FormValidationContainer as ɵe, VALIDATION_ERROR_CONFIG as ɵg, MapToMessagePipe as ɵh, InnerMapToMessagePipe as ɵi };
//# sourceMappingURL=xtream-ngx-validation-errors.js.map
